import os
from utils.input_output import load_yaml, get_groups_to_align_from_pg_yaml

checkpoint create_pg_fasta:
    input:
        fasta='seqs/all.faa',
        pg_yaml='pgs/protein_groups.yaml',
    output:
        directory('seqs/pgs')  # '/pg.{i}.faa'
    message:
        "creating FASTA files for protein groups ..."
    version:
        "0.3"
    run:
        os.mkdir(output[0])
        data = load_yaml(input.pg_yaml)
        sep = data['style']['protein_sep']
        pgs = data['protein_groups']
        group_ids = get_groups_to_align_from_pg_yaml(data)
        accs = []
        for group_id in group_ids:
            accs.append(set(pgs[group_id]['proteins'].split(sep)))

        started_groups = set()
        with open(input.fasta,"r") as inhandle:
            for line in inhandle:
                if len(line.strip()) == 0:
                    continue
                acc_line = line
                acc = acc_line[1:-1]
                seq = inhandle.readline()
                for acc_set_i, group_id in zip(accs,group_ids):
                    if acc in acc_set_i:
                        if not group_id in started_groups:
                            started_groups.add(group_id)
                            mode = "w"
                        else:
                            mode = "a"
                        with open(os.path.join(output[0],f'pg.{group_id}.faa'),mode) as outhandle:
                            outhandle.write(acc_line + seq)
